package com.scbfkj.uni.api;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.library.RequestUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.service.SecurityService;
import com.scbfkj.uni.system.Config;
import jakarta.annotation.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
public class SecurityApi {

    @Resource
    private SecurityService securityService;


    /**
     * 获取appToken appToken
     *
     * @param body
     * @return
     */
    @PostMapping("user/getToken")
    public ResponseEntity<Map<String, Object>> getToken(@RequestBody Map<String, Object> body) throws Exception {
        return ResponseEntity.ok(securityService.getToken(body));
    }

    /**
     * 刷新appToken
     *
     * @return
     */
    @PostMapping("user/refreshToken")
    public ResponseEntity<Map<String, Object>> refreshToken() throws Exception {
        return ResponseEntity.ok(securityService.refreshToken());
    }

    /**
     * 校验token 包含appToken 和用户token
     *
     * @return
     */
    @PostMapping({"user/testToken", "foxlibc/testToken"})
    public ResponseEntity<Map<String, Object>> testToken() throws Exception {
        String appToken = RequestUtil.getAppToken();
        return ResponseEntity.ok(securityService.verifyToken(appToken));
    }

    /**
     * 获取验证码
     *
     * @return
     */
    @PostMapping({"user/verifyCode", "foxlibc/verification-code"})
    public ResponseEntity<Map<String, Object>> getCode() throws Exception {
        return ResponseEntity.ok(securityService.verifyCode());
    }

    /**
     * 强制登录
     *
     * @return
     */
    @PostMapping({"user/forceLogin", "foxlibc/force_sign"})
    public ResponseEntity<Map<String, Object>> forceLogin() throws Exception {
        return ResponseEntity.ok(securityService.forceLogin());
    }

    /**
     * 登录
     *
     * @param body
     * @return
     */
    @PostMapping({"user/login", "foxlibc/sign-in"})
    public ResponseEntity<Map<String, Object>> login(@RequestBody Map<String, Object> body) throws Exception {
        return ResponseEntity.ok(securityService.login(body));
    }

    /**
     * 获取权限
     *
     * @return
     */
    @PostMapping({"user/permissions", "foxlibc/permissions"})
    public ResponseEntity<Map<String, Object>> getPermissions() throws Exception {
        return ResponseEntity.ok(securityService.permission());
    }


    /**
     * 更改密码
     *
     * @param body
     * @return
     */
    @PostMapping({"user/changePassword", "foxlibc/reset-passwd"})
    public ResponseEntity<Map<String, Object>> changePwd(@RequestBody Map<String, Object> body) throws Exception {
        return ResponseEntity.ok(securityService.changePassword(body));
    }

    /**
     * 登出注销
     *
     * @return
     */
    @PostMapping({"user/logOut", "foxlibc/sign-out"})
    public ResponseEntity<Map<String, Object>> logOut() throws Exception {
        return ResponseEntity.ok(securityService.logOut());
    }

    /**
     * 用户心跳检查   判断用户是否在线
     *
     * @return
     */
    @PostMapping({"user/health", "foxlibc/health"})
    public ResponseEntity<Map<String, Object>> health() throws Exception {
        return ResponseEntity.ok(securityService.userHeartbeat());
    }

    /**
     * 获取公钥字符串
     *
     * @return
     */
    @PostMapping("user/publicKey")
    public ResponseEntity<Map<String, Object>> getPubKey() {
        ObjectNode objectNode = new ObjectNode(JsonNodeFactory.instance);
        objectNode.put("pubKeyStr", DataEncryptionUtil.publicKeyStr);
        objectNode.put("type", DataEncryptionUtil.TYPE);
        objectNode.put("algorithm", DataEncryptionUtil.ALGORITHM);
        return ResponseEntity.ok(UniReturnUtil.success(objectNode));
    }
}
