package com.scbfkj.uni.library;

import org.springframework.core.io.ClassPathResource;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.security.SecureRandom;
import java.util.Base64;

public class ImageUtil {

    private static final int WIDTH = 120; // 图片宽度
    private static final int HEIGHT = 45; // 图片高度
    private static final String IMAGE_FORMAT = "JPG";
    private static final Color[] COLORS = new Color[]{
            new Color(0, 0, 255),
            new Color(0, 255, 0),
            new Color(255, 0, 0)
    };
    private static final SecureRandom RANDOM = new SecureRandom();


    public static String stringToImage(String code) throws IOException, FontFormatException {
        try (ByteArrayOutputStream outputStream = new ByteArrayOutputStream()) {
            BufferedImage image = createImage(code);
            ImageIO.write(image, IMAGE_FORMAT, outputStream);

            byte[] data = outputStream.toByteArray();
            return "data:image/" + IMAGE_FORMAT + ";base64," + Base64.getEncoder().encodeToString(data);
        }
    }

    private static BufferedImage createImage(String verifyCode) throws IOException, FontFormatException {
        BufferedImage image = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_RGB);
        Graphics2D graphics = image.createGraphics();

        graphics.setColor(Color.WHITE);
        graphics.fillRect(0, 0, WIDTH, HEIGHT);


        Font font = graphics.getFont();
        if (font == null) {
            File fontFile = new ClassPathResource("font/SourceHanSansSC-Regular-2.otf").getFile();
            font = Font.createFont(Font.TRUETYPE_FONT, fontFile);
        }
        font  = font.deriveFont((float) (HEIGHT - 10));
        graphics.setFont(font);

        int desX, desY=HEIGHT-10, distance = 18;
//        绘制验证码
        for (int i = 0; i < verifyCode.length(); i++) {
            graphics.setColor(COLORS[RANDOM.nextInt(COLORS.length)]);
            desX = i * distance+18;
            graphics.drawString(Character.toString(verifyCode.charAt(i)), desX,desY+ RANDOM.nextInt(desY%10));
        }
//        绘制干扰线
        for (int x = 0; x < 6; x++) {
            graphics.setColor(getRandomColor());
            graphics.drawLine(RANDOM.nextInt(WIDTH), RANDOM.nextInt(HEIGHT), RANDOM.nextInt(WIDTH), RANDOM.nextInt(HEIGHT));
        }

        graphics.dispose();
        return image;
    }

    private static Color getRandomColor() {
        int r = RANDOM.nextInt(256);
        int g = RANDOM.nextInt(256);
        int b = RANDOM.nextInt(256);
        return new Color(r, g, b);
    }
}
