package com.scbfkj.uni.library;

import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.springframework.web.context.request.RequestAttributes.SCOPE_REQUEST;
import static org.springframework.web.context.request.RequestAttributes.SCOPE_SESSION;

public class RequestUtil {

    public static String getIpAddr() {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.isEmpty() || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.isEmpty() || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.isEmpty() || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    public static String getAppToken() {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
        Object appToken = requestAttributes.getRequest().getHeader("token");
        return Objects.nonNull(appToken) ? appToken.toString() : null;

    }

    public static String getUserToken() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
        Object userToken = requestAttributes.getRequest().getHeader("usertoken");
        return Objects.nonNull(userToken) ? userToken.toString() : null;

    }

    public static String getAppId() throws Exception {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
        Object appid = requestAttributes.getAttribute("appid", SCOPE_SESSION);
        if(Objects.nonNull(appid)) {
            return  appid.toString();
        }
        return getApplication().get("appid").toString();

    }

    public static Map<String, Object> getApplication() throws Exception {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
        Object application = requestAttributes.getAttribute("application", SCOPE_SESSION);
        if (Objects.isNull(application)) {
            String appToken = getAppToken();
            String requestIp = getIpAddr();
            String query = "select * from appconnectlog where apptoken=? and requestip =?";
            List<Map<String, Object>> appConnectLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
                add(new Object[]{appToken, requestIp});
            }});
            if (appConnectLogList.isEmpty()) {
                throw new RuntimeException("当前连接未登录");
            }
            Map<String, Object> stringObjectMap = appConnectLogList.get(0);
            Object applicationid = stringObjectMap.get("appid");

            query = "select * from application where appid=?";
            List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
                add(new Object[]{applicationid});
            }});
            if (applicationList.isEmpty()) {
                throw new RuntimeException("获取应用失败");
            }
            return applicationList.get(0);
        }
        return (Map<String, Object>) application;


    }

    public static String getUserId() {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
        Object appid = requestAttributes.getAttribute("userid", SCOPE_SESSION);
        return Objects.nonNull(appid) ? appid.toString() : null;

    }

    public static Map<String, Object> getUserInfo() {
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
        Object userInfo = requestAttributes.getAttribute("userinfo", SCOPE_SESSION);
        return Objects.nonNull(userInfo) ? (Map<String, Object>) userInfo : null;
    }

    public static String getSessionId() {
        return RequestContextHolder.currentRequestAttributes().getSessionId();
    }

    public static String getUri() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
        return requestAttributes.getRequest().getRequestURI();
    }
}
