package com.scbfkj.uni.library.script;

import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.service.LoggerService;
import org.apache.commons.pool2.BaseKeyedPooledObjectFactory;
import org.apache.commons.pool2.KeyedObjectPool;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.impl.DefaultPooledObject;
import org.apache.commons.pool2.impl.GenericKeyedObjectPool;
import org.apache.commons.pool2.impl.GenericKeyedObjectPoolConfig;
import org.graalvm.polyglot.Context;
import org.graalvm.polyglot.Source;
import org.graalvm.polyglot.Value;

import java.time.LocalDateTime;
import java.util.Map;
import java.util.Objects;

public final class JsScriptEngineUtil {

    private final static String TO_STRING_SCRIPT = """
            (value) => {
                if (typeof value === 'string') return value
                return JSON.stringify(value);
            }
            """;


    private final static KeyedObjectPool<String, Value> scriptPool = new GenericKeyedObjectPool<>(new BaseKeyedPooledObjectFactory<String, Value>() {
        @Override
        public Value create(String script) throws Exception {
            return Context.create("js").eval(Source.create("js", script));
        }

        @Override
        public PooledObject<Value> wrap(Value value) {
            return new DefaultPooledObject<>(value);
        }

        @Override
        public void destroyObject(String key, PooledObject<Value> p) throws Exception {
            super.destroyObject(key, p);
        }
    }, new GenericKeyedObjectPoolConfig<>());


    public static Map<String,Object> eval(String script, Object... args) {
        Value function = null;
        try {
        function = scriptPool.borrowObject(script);
            if (function.canExecute()) {
                Value result = function.execute(args);
                return UniReturnUtil.success(toString(result));
            } else {
                return UniReturnUtil.success(toString(function));
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            if (Objects.nonNull(function)) {
                try {
                    scriptPool.returnObject(script, function);
                } catch (Exception e) {
                    LoggerService.logSystemError(LocalDateTime.now(), null, UniReturnUtil.getMessage(e));
                }
            }
        }
    }

    private static String toString(Value value) throws Exception {
        Value function = scriptPool.borrowObject(TO_STRING_SCRIPT);
        try {
            return function.execute(value).asString();
        } finally {
            scriptPool.returnObject(TO_STRING_SCRIPT, function);
        }
    }
}
