package com.scbfkj.uni.process;

import com.fasterxml.jackson.databind.JsonNode;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.script.DatabaseScriptUtil;
import com.scbfkj.uni.system.Config;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.pool.HikariPool;
import jakarta.annotation.Nonnull;

import java.sql.*;
import java.time.LocalDateTime;
import java.util.*;

public class DataBase {

    private final static Map<String, Map<String, Object>> cacheDatas = new HashMap<>();
    private  static List<Map<String, Object>> cacheConfigList = new ArrayList<>();
    public final static Map<String, HikariPool> dataSourcePools = new HashMap<>();


    public static List<Map<String, Object>> query(String connectionStr, String sql, List<Object[]> argsList) throws Exception {

//        缓存数据 connectionStr + sql

        String key = "%s;%s;%s".formatted(connectionStr, sql, DataFormatUtil.toString(argsList));
        if (cacheDatas.containsKey(key)) {
            Map<String, Object> cacheData = cacheDatas.get(key);

            Object effectiveduration = cacheData.get("effectiveduration");
            if (Objects.isNull(effectiveduration) || ((LocalDateTime) effectiveduration).isBefore(LocalDateTime.now())) {
                cacheDatas.remove(key);
            } else {
                return ((List<Map<String, Object>>) cacheData.get("data"));
            }

        }

        HikariPool dataSourcePool = getDataSourcePool(connectionStr);
        List<Map<String, Object>> result = argsList.parallelStream().flatMap(args -> {
            try (Connection connection = dataSourcePool.getConnection();
                 PreparedStatement preparedStatement = connection.prepareStatement(sql)
            ) {
                for (int i = 0; i < args.length; i++) {
                    preparedStatement.setObject(i + 1, args[i]);
                }
                ResultSet resultSet = preparedStatement.executeQuery();
                return getResult(connectionStr, sql, resultSet).stream();
            } catch (Exception exception) {
                throw new RuntimeException("数据异常: %s\n sql: %s ;\n args: %s ".formatted(exception.getMessage(), sql, DataFormatUtil.toDate(args)));
            }
        }).toList();

            String querySql = "select * from datacache where 1=?";
        if ( !Objects.equals(querySql,sql)) {
            cacheConfigList=  query(Config.centerConnectionStr,querySql,Collections.singletonList(new Object[]{1}));
        }
        cacheConfigList.stream().filter(it -> Objects.equals(it.get("querysql"), sql) && Objects.equals(it.get("connectset"), connectionStr)).findFirst().ifPresent(it -> {
            Object o = it.get("effectiveduration");
            LocalDateTime localDateTime = LocalDateTime.now().plusSeconds(Long.parseLong(o.toString()));
            cacheDatas.put(key, new HashMap<>() {{
                put("effectiveduration", localDateTime);
                put("data", result);
            }});
        });
        return result;
    }


    public static int[] updateBatch(String connectionStr, String sql, List<Object[]> argsList) throws Exception {
        HikariPool dataSourcePool = getDataSourcePool(connectionStr);
        try (Connection connection = dataSourcePool.getConnection();
             PreparedStatement preparedStatement = connection.prepareStatement(sql)
        ) {
            int[] result;
            int index = 0;
            try {
                while (argsList.size() > index) {
                    Object[] args = argsList.get(index);
                    for (int i = 0; i < args.length; i++) {
                        preparedStatement.setObject(i + 1, args[i]);
                    }
                    preparedStatement.addBatch();
                    index++;
                }
                result = preparedStatement.executeBatch();
                connection.commit();
            } catch (SQLException e) {
                connection.rollback();
                throw new RuntimeException(e.getMessage());
            }

            return result;
        }
    }

    public static int exec(String connectionStr, String sql) throws Exception {
        HikariPool dataSourcePool = getDataSourcePool(connectionStr);
        try (Connection connection = dataSourcePool.getConnection();
             PreparedStatement preparedStatement = connection.prepareStatement(sql)
        ) {
            int i = preparedStatement.executeUpdate();
            try {
                connection.commit();
            } catch (Exception e) {
                connection.rollback();
                throw e;
            }
            return i;
        }
    }



    public static List<Map<String, Object>> query(String connectionStr, String sql, List<Map<String, Object>> argsList, List<String> filterColumns, List<Map<String, Object>> filterLines) throws Exception {

        sql = sql.replaceAll("(\\r)?\\n", " ");
        String filterLineWhereStr =null;
        for (Object f : filterLines) {
            Map<String, Object> it = ((Map<String, Object>) f);
            filterLineWhereStr = " %s %s %s %s %s %s %s %s ".formatted(
                    filterLineWhereStr,
                    it.getOrDefault("left", ""),
                    it.get("column"),
                    it.get("comparator"),
                    Objects.equals(it.get("comparator"), " is null ") ? "" : "?",
                    !Objects.equals(it.get("comparator"), " is null ")?" ":it.get("value"),
                    it.getOrDefault("right", ""),
                    it.getOrDefault("connector", "")
            );
        }
        if (Objects.nonNull(filterLineWhereStr) ) {
            sql = " %s %s and %s ".formatted(sql, sql.contains(" where ") ? " 1 = 1" : " where ", Objects.isNull(filterLineWhereStr)?" 1=1 ":filterLineWhereStr);
        }
        if (Objects.nonNull(filterColumns) && !filterColumns.isEmpty()) {
            sql = "select %s from ( %s ) T".formatted(String.join(",", filterColumns), sql);
        }

        List<List<Object>> result = new ArrayList<>();

        List<String> names = DatabaseScriptUtil.getSQLVarList(sql);
        String newSql = DatabaseScriptUtil.sqlStrNewSQL.get(sql);

        if (names.size() == 1 && names.contains("whereStr")) {
            List<Map<String, Object>> list = new ArrayList<>();
            for (Map<String, Object> stringObjectMap : argsList) {
                List<Object> args = new ArrayList<>();
                StringJoiner joiner = new StringJoiner(" and ");
                for (Map.Entry<String, Object> entry : stringObjectMap.entrySet()) {
                    String formatted = "%s = ?".formatted(entry.getKey());
                    args.add(entry.getValue());
                    joiner.add(formatted);
                }
                String whereStr = " " + joiner;
                String sqlStr = sql.replace("《whereStr》", whereStr);
                List<Map<String, Object>> apply = query(connectionStr, sqlStr, new ArrayList<>() {{
                    add(args.toArray());
                }});
                list.addAll(apply);
            }
            return list;
        } else {
            for (Map<String, Object> map : argsList) {
                List<Object> args = new ArrayList<>();
                map= ((Map<String,Object>) map.getOrDefault("filter", map));
                for (String name : names) {
                    args.add(map.get(name));
                }
                result.add(args);
            }
            return query(connectionStr, newSql, result.stream().map(List::toArray).toList());
        }
    }


    private static HikariPool getDataSourcePool(String connectionStr) throws Exception {
        if (dataSourcePools.containsKey(connectionStr)) {
            return dataSourcePools.get(connectionStr);
        }
        JsonNode jsonNode = DataFormatUtil.toJsonNode(connectionStr);

        JsonNode jdbcUrl = jsonNode.get("jdbcUrl");
        JsonNode username = jsonNode.get("username");
        JsonNode password = jsonNode.get("password");
        JsonNode driverClassName = jsonNode.get("driverClassName");
        HikariPool dataSourcePool = createDataSourcePool(jdbcUrl.asText(), Objects.isNull(username) ? null : username.asText(), Objects.isNull(password) ? null : password.asText(), Objects.isNull(driverClassName) ? null : driverClassName.asText(), connectionStr);
        dataSourcePools.put(connectionStr, dataSourcePool);
        return dataSourcePool;
    }


    private static HikariPool createDataSourcePool(@Nonnull String url, String username, String password, String driver, String poolName) throws Exception {
        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setMaximumPoolSize(Math.min(Runtime.getRuntime().availableProcessors(), 8));//最大核心数:当前可用CPU数，最大16
        hikariConfig.setKeepaliveTime(60000); //用于跟数据库保持心跳连接
        hikariConfig.setMaxLifetime(60000 * 60 * 4); //4小时 接在池中的最大生存时间，超过该时间强制逐出
        hikariConfig.addDataSourceProperty("cachePrepStmts", "true"); //是否自定义配置，为true时下面两个参数才生效
        hikariConfig.addDataSourceProperty("prepStmtCacheSize", "250"); //连接池大小默认25，官方推荐250-500
        hikariConfig.addDataSourceProperty("prepStmtCacheSqlLimit", "2048"); //单条语句最大长度默认256，官方推荐2048
        hikariConfig.addDataSourceProperty("useServerPrepStmts", "true"); //新版本MySQL支持服务器端准备，开启能够得到显著性能提升
        hikariConfig.addDataSourceProperty("useLocalSessionState", "true");
        hikariConfig.addDataSourceProperty("useLocalTransactionState", "true");
        hikariConfig.addDataSourceProperty("rewriteBatchedStatements", "true");//批量新增
        hikariConfig.addDataSourceProperty("cacheResultSetMetadata", "true");
        hikariConfig.addDataSourceProperty("cacheServerConfiguration", "true");
        hikariConfig.addDataSourceProperty("elideSetAutoCommits", "true");
        hikariConfig.addDataSourceProperty("maintainTimeStats", "false");

        hikariConfig.setJdbcUrl(url);

        if (Objects.nonNull(username)) {
            hikariConfig.setUsername(username);
        }
        if (Objects.nonNull(password)) {
            hikariConfig.setPassword(password);
        }
        if (Objects.nonNull(driver)) {
            hikariConfig.setDriverClassName(driver);
        }
        if (Objects.nonNull(poolName)) {
            hikariConfig.setPoolName(poolName);
        }
        hikariConfig.setAutoCommit(false);
        return new HikariPool(hikariConfig);
    }

    private static final Map<String, List<String>> columns = new HashMap<>();

    private static List<String> getColumns(String connection, String sql, ResultSet resultSet) throws SQLException {
        if (columns.containsKey(connection + sql)) {
            return columns.get(connection + sql);
        }
        ArrayList<String> names = new ArrayList<>();
        ResultSetMetaData metaData = resultSet.getMetaData();
        for (int i = 0; i < metaData.getColumnCount(); i++) {
            names.add(metaData.getColumnName(i + 1));
        }
        columns.put(connection + sql, names);
        return names;
    }

    public static List<Map<String, Object>> getResult(String connection, String sql, ResultSet resultSet) throws SQLException {
        List<String> cs = getColumns(connection, sql, resultSet);
        List<Map<String, Object>> result = new ArrayList<>();
        while (resultSet.next()) {
            Map<String, Object> resultMap = new HashMap<>();
            for (String name : cs) {
                Object object = resultSet.getObject(name);
                resultMap.put(name, object);
            }
            result.add(resultMap);
        }
        return result;
    }


}
