package com.scbfkj.uni.service;

import com.scbfkj.uni.library.DataAliasGetUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import com.scbfkj.uni.system.ScheduleUtil;

import java.util.*;

public class ControlService {


    public static void startServiceByContainerCode() throws Exception {

//        查询服务类型为主动采集的服务
        String queryServiceInfos = "select * from serviceinfo where containercode=? and servicetype=4";
        List<Map<String, Object>> serviceInfos = DataBase.query(Config.centerConnectionStr, queryServiceInfos, Collections.singletonList(new Object[]{Config.containerCode}));
//        循环启动
        for (Map<String, Object> serviceInfo : serviceInfos) {
            start(serviceInfo);
        }
    }

    /**
     * 服务启动 主要用于前端或者第三方请求
     *
     * @param inData
     * @return
     */
    public static Map<String, Object> start(Map<String, Object> inData) throws Exception {
        Optional<String> serviceidOpt = DataAliasGetUtil.getValue("serviceid", inData);
        if (serviceidOpt.isEmpty()) {
            throw new RuntimeException("服务编号不能为空");
        }
        String serviceId = serviceidOpt.get();
        List<Map<String, Object>> serviceInfoList = null;
        if (serviceId.matches("^\\d+$")) {
            serviceInfoList = DataBase.query(Config.centerConnectionStr, "select * from serviceinfo where serviceid=?", Collections.singletonList(new Object[]{serviceId}));
        } else {
            serviceInfoList = DataBase.query(Config.localCenterConnectionStr, "select * from serviceinfo where serviceid=?", Collections.singletonList(new Object[]{serviceId}));
        }
        if (serviceInfoList.isEmpty()) {
            throw new RuntimeException("服务%s没有配置".formatted(serviceId));
        }
        Map<String, Object> serviceInfo = serviceInfoList.get(0);
        Object cronexpress = serviceInfo.get("cronexpress");
        if (Objects.isNull(cronexpress)) {
            return DataProcessService.process(inData);

        } else {
            //启动定时任务：
            boolean start = ScheduleUtil.start(serviceId);
            if (start) {
                DataBase.updateBatch(Config.centerConnectionStr, "update serviceinfo set runState = 1 where  serviceid =?", Collections.singletonList(new Object[]{serviceId}));
                return UniReturnUtil.success("启动成功");
            } else {
                return UniReturnUtil.fail("启动失败");
            }
        }
    }


    /**
     * 服务停止
     *
     * @param serviceId
     * @return
     */
    public static Map<String, Object> stop(String serviceId) throws Exception {
        boolean cancel = ScheduleUtil.cancel(serviceId);
        DataBase.updateBatch(Config.centerConnectionStr, "update serviceinfo set runState = 0 where  serviceID =?", Collections.singletonList(new Object[]{serviceId}));
//        LogUtils.log("stop: 2", "0", null, (reset ? "停止成功" : "停止失败"), serviceId, null, null, null,null);//无条件记录数据接收日志
        return UniReturnUtil.success("停止" + (cancel ? "成功" : "失败"));
    }

    /**
     * 用用户启停定时任务 服务类型为4的服务
     *
     * @param params
     * @param statue
     * @return
     * @throws Exception
     */

    public static Map<String, Object> startOrStop(Map<String, Object> params, String statue) throws Exception {
        Optional<String> serviceIdOpt = DataAliasGetUtil.getValue("serviceid", params);
        if (serviceIdOpt.isEmpty()) return UniReturnUtil.fail("服务ID不能为空");
        String serviceId = serviceIdOpt.get();
        Map<String, Object> stop = stop(serviceId);
        if (statue.equals("0")) {
            return stop;
        }
        List<Map<String, Object>> serviceTypeData = DataBase.query(Config.centerConnectionStr, "select serviceType from serviceinfo where serviceID=? and containercode=? and servicetype='4'", Collections.singletonList(new Object[]{serviceId, Config.containerCode}));
        if (serviceTypeData.isEmpty()) {
            throw new RuntimeException("没有找到服务配置");
        }
        return start(new HashMap<>() {{
            put("service", serviceId);
        }});
    }
}
