package com.scbfkj.uni.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.jayway.jsonpath.JsonPath;
import com.scbfkj.uni.library.DataAliasGetUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.library.script.DatabaseScriptUtil;
import com.scbfkj.uni.library.script.JavaScriptEngineUtil;
import com.scbfkj.uni.library.script.JsScriptEngineUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;


public class DataProcessService {


    public static Map<String, Object> process(Map<String, Object> inData) {
        String lifecycleid = null;
        List<Map<String, Object>> resource = new ArrayList<>();
        List<Map<String, Object>> preResource = new ArrayList<>();
        resource.add(inData);
        String serviceId = null;
        LocalDateTime startDateTime = LocalDateTime.now();
        Optional<String> serviceIdOpt;
        String message = null;
        Object algorithmlibraryid;

        try {
            serviceIdOpt = DataAliasGetUtil.getValue("serviceid", inData);
            if (Objects.isNull(inData) || inData.isEmpty() || serviceIdOpt.isEmpty())
                throw new RuntimeException("服务编号不能为空");
            serviceId = serviceIdOpt.get();

//        熔断

            List<Map<String, Object>> algorithmLibraries;
            if (serviceId.matches("^\\d+$")) {
                algorithmLibraries = DataBase.query(Config.centerConnectionStr, "select * from algorithmlibrary where serviceid=?", Collections.singletonList(new Object[]{serviceId}));
            } else {
                algorithmLibraries = DataBase.query(Config.localCenterConnectionStr, "select * from algorithmlibrary where serviceid=?", Collections.singletonList(new Object[]{serviceId}));
            }

            Optional<String> lifecycleidOpt;
            lifecycleidOpt = DataAliasGetUtil.getValue("lifecycleid", inData);

            if (lifecycleidOpt.isEmpty()) {
                lifecycleid = DataAliasGetUtil.createLifeCycleCol(Config.containerCode, serviceId);
                inData.put("lifecycleid", lifecycleid);
            } else {
                lifecycleid = lifecycleidOpt.get();
            }

            for (Map<String, Object> algorithmLibrary : algorithmLibraries) {

                Map<String, Object> algorithmResult = null;

                long execTime;

                startDateTime = LocalDateTime.now();
                long startTime = System.currentTimeMillis();
                List<Object> parameters = null;
                algorithmlibraryid = algorithmLibrary.get("algorithmlibraryid");
                Object preConditions = algorithmLibrary.get("preconditions");
                Object preparameterset = algorithmLibrary.get("preparameterset");
                HashMap<String, Object> data = new HashMap<>();
                resource.add(data);
//                记录生命周期ID
                data.put("lifecycleid", lifecycleid);
                data.put("algorithmlibraryid", algorithmlibraryid);
                HashMap<String, Object> source = new HashMap<>();
                source.put("args", resource);
                source.put("algorithm", algorithmLibrary);
                if (Objects.nonNull(preConditions)) {
                    HashMap<String, Object> preData = new HashMap<>();
                    preResource.add(preData);

                    preData.put("algorithmlibraryid", algorithmlibraryid);


                    String preCode;
                    List<Object> params = getParams(Optional.ofNullable(preparameterset).map(DataFormatUtil::toString).orElse(null), source);
                    Map<String, Object> eval = JsScriptEngineUtil.eval(DataFormatUtil.toString(preConditions), params.toArray());

                    preData.put("preResult", eval);
                    if (!Objects.equals(eval.get("code"), "0"))
                        throw new RuntimeException(eval.get("message").toString());
                    preCode = eval.get("returnData").toString();


                    preData.put("preCode", preCode);
//                    记录前置条件结果
                    preData.put("preParameters", params);
                    if (Objects.equals("2", preCode)) {
                        break;
                    } else if (Objects.equals("1", preCode)) {
                        resource.add(null);
                        continue;
                    }
                }

                Object parameterSet = algorithmLibrary.get("parameterset");
                Object dataSourceId = algorithmLibrary.get("datasourceid");
                List<Map<String, Object>> datasourceList = DataBase.query(Config.centerConnectionStr, "select * from datasource where datasourceid=?", Collections.singletonList(new Object[]{dataSourceId}));
                Map<String, Object> datasource = datasourceList.get(0);

//        获取入参列表
                parameters = new ArrayList<>();
                if (Objects.nonNull(parameterSet)) {
                    source.put("datasource", datasource);
                    parameters.addAll(getParams(parameterSet.toString(), source));
//                    算法入参
                    data.put("parameters", parameters);

                }
                algorithmResult = processByAlgorithm(algorithmLibrary, parameters);
//                    算法执行结果
                data.put("result", algorithmResult);
                execTime = System.currentTimeMillis() - startTime;
//                    执行时长
                data.put("execTime", execTime);
//                    执行成功与否
                Object code = Optional.ofNullable(algorithmResult.get("code")).map(DataFormatUtil::toString).orElse(null);
                message = Optional.ofNullable(algorithmResult.get("message")).map(DataFormatUtil::toString).orElse(null);
                data.put("resultCode", code);

                if (!Objects.equals(code, "0")) {
                    throw new RuntimeException(message);
                }
            }
            return UniReturnUtil.success(((Map) resource.get(resource.size() - 1).get("result")).get("returnData"));
        } catch (Exception e) {
            message = e.getMessage();
            return UniReturnUtil.fail(e);
        } finally {
//                    不管成功还是失败都记录日志
            LoggerService.logService(
                    startDateTime,
                    serviceId,
                    DataFormatUtil.toString(resource),
                    DataFormatUtil.toString(preResource),
                    LocalDateTime.now(),
                    Objects.nonNull(message) ? "-1" : "0",
                    message,
                    lifecycleid
            );
        }

    }

    /**
     * @param algorithmLibrary 算法配置
     * @param parameters       算法参数
     * @return
     * @throws Exception
     */
    public static Map<String, Object> processByAlgorithm(Map<String, Object> algorithmLibrary, List<Object> parameters) throws Exception {

        Object type = algorithmLibrary.get("algorithmtype");
        Object expression = algorithmLibrary.get("computingexpression");


        switch (type.toString()) {
//            java反射
            case "1" -> {
                return JavaScriptEngineUtil.invoke((Map<String, Object>) parameters.get(0), expression.toString(), parameters.subList(1,parameters.size()));
            }
//            JS表达式
            case "2" -> {
                return JsScriptEngineUtil.eval(expression.toString(), parameters);
            }
//            数据库
            case "3" -> {
//                下放到Database中处理数据
//                参数表达式顺序是 数据源连接字符串（String.$.datasource.connectset）,sql表达式(String.$.algorithm.computingexpression)，需要操作的值(List.$.args[1].returnData)，执行编号(String.$.algorithm.executionnumber)
                return DatabaseScriptUtil.exec(parameters.get(0).toString(), parameters.get(1).toString(), ((List<Map<String, Object>>) parameters.get(2)), parameters.get(3), (List<String>) parameters.get(4), (List<Map<String, Object>>) parameters.get(5));
            }
        }
        return UniReturnUtil.fail("算法类型不支持");
    }


    /**
     * @param parameterSet 使用非标准的jsonPath表达式 String.$.a.b[0].c,String.abcc 多个参数之间使用;;分隔
     * @param source       jsonpath需要取值的数据源
     * @return
     */
    public static List<Object> getParams(String parameterSet, Object source) {
        if (Objects.isNull(parameterSet)) return Collections.emptyList();
        String[] paths = parameterSet.split(";;");
        String json = DataFormatUtil.toString(source);
        List<Object> result = new ArrayList<>();

        for (String it : paths) {
            it = it.trim();
            String type = it.split("\\.")[0];
            if(!List.of("string","list","map","array","long","integer","boolean","double","float","datetime").contains(type.toLowerCase())){
                if(Objects.equals("null",type)) {
                    result.add(null);
                }else{
                    result.add(type);
                }
                    continue;
            }
            String path = it.replace(type + ".", "");
            Object value = null;
//            $开头的使用标准的jsonPath
            if (path.startsWith("$")) {
                value = JsonPath.read(json, path);
            } else if (path.startsWith("#")) {
//                todo  缓存之类的特殊值

//                不是使用$#开头的直接取后面的字符串
            } else {
                value = path;
            }
            if (Objects.isNull(value)) {
                result.add(null);
                continue;
            }
            if ("String".equalsIgnoreCase(type)) {
                value = DataFormatUtil.toString(value);
            } else if ("List".equalsIgnoreCase(type)) {
                value = DataFormatUtil.toList(value);
            } else if ("Array".equalsIgnoreCase(type)) {
                value = DataFormatUtil.toArray(value);
            } else if ("Map".equalsIgnoreCase(type)) {
                value = DataFormatUtil.toMap(value);
            } else if ("Long".equalsIgnoreCase(type)) {
                value = Long.parseLong(value.toString());
            } else if ("Integer".equalsIgnoreCase(type)) {
                value = Integer.parseInt(value.toString());
            } else if ("Double".equalsIgnoreCase(type)) {
                value = Double.parseDouble(value.toString());
            } else if ("Float".equalsIgnoreCase(type)) {
                value = Float.parseFloat(value.toString());
            } else if ("Boolean".equalsIgnoreCase(type)) {
                value = Boolean.parseBoolean(value.toString());
            } else if ("Datetime".equalsIgnoreCase(type)) {
                String string = value.toString();
                String patten = "yyyy-MM-dd HH:mm:ss";
                if (string.contains("(")) {
                    patten = string.substring(string.indexOf("(") + 1, string.length() - 1);
                    string = string.substring(0, string.indexOf("("));
                }
                value = LocalDateTime.parse(string, DateTimeFormatter.ofPattern(patten));
            }

            result.add(value);
        }
        return result;
    }


}
