package com.scbfkj.uni.service;

import com.scbfkj.uni.library.*;
import com.scbfkj.uni.library.script.DatabaseScriptUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.util.*;

import static com.scbfkj.uni.library.DataAliasGetUtil.getValue;
import static org.springframework.web.context.request.RequestAttributes.SCOPE_REQUEST;
import static org.springframework.web.context.request.RequestAttributes.SCOPE_SESSION;

@Service
public class SecurityService {


    @Value("${app.token-effective:604800}")
    private long defaultAppTokenEffective;
    @Value("${app.code-effective:600}")
    private long defaultSecurityCodeEffective;

    @Value("${app.debug:false}")
    private boolean debug;


    private static final SecureRandom RANDOM = new SecureRandom();

    private static String createCode(int size, String source) {
        StringBuilder verifyCode = new StringBuilder(size);
        int codesLen = source.length();
        for (int i = 0; i < size; i++) {
            verifyCode.append(source.charAt(RANDOM.nextInt(codesLen - 1)));
        }
        return verifyCode.toString();
    }


    //安全类服务
    //连接认证--获取连接令牌

    public Map<String, Object> getToken(Map<String, Object> requestData) throws Exception {
        Optional<String> appid = getValue("appid", requestData);
        Optional<String> appSecret = getValue("appsecret", requestData);
        if (appSecret.isPresent() && appid.isPresent()) {
            String clean = "delete from appconnectlog where expiretime < ? ";
            DataBase.updateBatch(Config.securityConnectionStr, clean, new ArrayList<>() {{
                add(new Object[]{LocalDateTime.now()});
            }});
            String query = "select * from application where appid = ? and appsecret = ?";
            List<Map<String, Object>> applicationList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
                add(new Object[]{appid.get(), appSecret.get()});
            }});

            if (applicationList.isEmpty()) {
                return UniReturnUtil.fail("appid 或 appsecret 错误");
            }
            Map<String, Object> application = applicationList.get(0);


            Object apptokeneffectiveObj = application.get("apptokeneffective");
            Long apptokeneffective = defaultAppTokenEffective;
            if (Objects.nonNull(apptokeneffectiveObj)) {
                apptokeneffective = Long.parseLong(apptokeneffectiveObj.toString());
            }
            LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(apptokeneffective);
            Map<String, Object> data = new HashMap<>();
            String ip = RequestUtil.getIpAddr();
            List<Map<String, Object>> logs = DataBase.query(Config.securityConnectionStr, "select * from appconnectlog where requestip=? and appid = ?", Collections.singletonList(new Object[]{ip, appid.get()}));
            String appToken;
            if (!logs.isEmpty()) {
                DataBase.updateBatch(Config.securityConnectionStr, """
                                                update appconnectlog
                        set expiretime = ?,lasttime=?
                        where connid=?""", Collections.singletonList(new Object[]{expiresTime, LocalDateTime.now(), logs.get(0).get("connid")}));
                Map<String, Object> map = logs.get(0);
                RequestContextHolder.currentRequestAttributes().setAttribute("application", application, SCOPE_SESSION);
                appToken = map.get("apptoken").toString();
            } else {
                String sessionId = RequestUtil.getSessionId();
                appToken = DataEncryptionUtil.signatureMD5("%s:%s".formatted(LocalDateTime.now(), sessionId));
                RequestContextHolder.currentRequestAttributes().setAttribute("appid", appid, SCOPE_SESSION);

                DataBase.updateBatch(Config.securityConnectionStr, """
                                                insert into appconnectlog (appid, requesttime, requestip, apptoken, expiretime, lasttime)
                        values (?,?,?,?,?,?)""", Collections.singletonList(new Object[]{appid.get(), LocalDateTime.now(), ip, appToken, expiresTime, LocalDateTime.now()}));

            }
            data.put("token", appToken);
            data.put("expirestime", DataFormatUtil.toString(expiresTime));
            data.put("appname", application.get("appname"));
            data.put("appenname", application.get("appengname"));
            data.put("logo", application.get("applog"));
            data.put("smalllogo", application.get("smalllogo"));
            data.put("background", application.get("backgroundimage"));
            data.put("securitycoderule", application.get("securitycoderule"));
            RequestContextHolder.currentRequestAttributes().setAttribute("application", application, SCOPE_SESSION);
            return UniReturnUtil.success(data);
        }
        return UniReturnUtil.fail("appid 或 appsecret 不能为空");
    }

    //校验连接令牌

    public Map<String, Object> verifyToken(String appToken) throws Exception {
        String clean = "delete from appconnectlog where expiretime <  ?";

        DataBase.updateBatch(Config.securityConnectionStr, clean, new ArrayList<>() {{
            add(new Object[]{LocalDateTime.now()});
        }});

        String requestIp = RequestUtil.getIpAddr();
        String query = "select * from appconnectlog where apptoken=? and requestip =?";
        List<Map<String, Object>> appConnectLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{appToken, requestIp});
        }});
        Map<String, Object> data = new HashMap<>();
        if (!appConnectLogList.isEmpty()) {
            Map<String, Object> appConnectLog = appConnectLogList.get(0);
            data.put("validstatus", true);
            data.put("appid", appConnectLog.get("appid"));
            return UniReturnUtil.success(data);
        }
        data.put("validstatus", false);
        return UniReturnUtil.success(data);
    }

    //刷新连接令牌

    public Map<String, Object> refreshToken() throws Exception {

        RequestAttributes requestAttributes = RequestContextHolder.currentRequestAttributes();
        Object appid = requestAttributes.getAttribute("appid", SCOPE_REQUEST);

        Map<String, Object> application = RequestUtil.getApplication();

        Long apptokeneffective = defaultAppTokenEffective;
        Object apptokeneffectiveObj = application.get("apptokeneffective");

        if (Objects.nonNull(apptokeneffectiveObj)) {
            apptokeneffective = Long.parseLong(apptokeneffectiveObj.toString());
        }
        LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(apptokeneffective);
        Object appToken = RequestUtil.getAppToken();
        Map<String, Object> data = new HashMap<>();
        try {
            String update = "update appconnectlog set expiretime=? where apptoken =? and requestip =? and appid=?";
            DataBase.updateBatch(Config.securityConnectionStr, update, new ArrayList<>() {{
                add(new Object[]{expiresTime, appToken, RequestUtil.getIpAddr(), appid});
            }});
            data.put("expirestime", DataFormatUtil.toString(expiresTime));
            data.put("token", appToken);
            return UniReturnUtil.success(data);
        } catch (Exception exception) {
            return UniReturnUtil.fail("刷新令牌失败：%s".formatted(exception.getMessage()));
        }
    }

    //获取登录验证码

    public Map<String, Object> verifyCode() throws Exception {

        String appId = RequestUtil.getAppId();
        Map<String, Object> application = RequestUtil.getApplication();
        Object securityCodeRule = application.get("securitycoderule");
        Object securityCodeEffectiveObj = application.get("securitycodeeffective");
        if (Objects.nonNull(securityCodeRule)) {
            String rule = "";
            String codeRule = (String) securityCodeRule;
//                数字
            if (codeRule.contains("N")) {
                rule += "23456789";
            }
//            大小写字母 world
            if (codeRule.contains("W")) {
                rule += "ABCDEFGHGKMNPQRSTUVWXYZ";
                rule += "abcdefghgkmnpqrstuvwxyz";
//                大写字母 upperCase
            } else if (codeRule.contains("U")) {
                rule += "ABCDEFGHGKMNPQRSTUVWXYZ";
//                小写字母 lower
            } else if (codeRule.contains("L")) {
                rule += "abcdefghgkmnpqrstuvwxyz";
            }
//            字符 c
            if (codeRule.contains("C")) {
                rule += "~!@#%&-_;:";
            }
//            最后四位长度为最小长度和最大长度
            String minMax = codeRule.substring(codeRule.length() - 4);

            Integer min = Integer.parseInt(minMax.substring(0, 2));
            Integer max = Integer.parseInt(minMax.substring(2));

            Integer length = min;
            if (!Objects.equals(min, max)) {
                Random random = new Random();
                do {
                    length = random.nextInt(max);
                } while (length <= min);
            }
            String code = createCode(length, rule);
            String base64Code = ImageUtil.stringToImage(code);
            String sessionId = RequestUtil.getSessionId();
            String ip = RequestUtil.getIpAddr();
            Long securityCodeEffective = defaultSecurityCodeEffective;
            if (Objects.nonNull(securityCodeEffectiveObj)) {
                securityCodeEffective = Long.parseLong(securityCodeEffectiveObj.toString());
            }
            addCode(code, sessionId, appId, securityCodeEffective, ip);

            Map<String, Object> data = new HashMap<>();
            data.put("verifyCodeImage", base64Code);
            if (debug)
                data.put("code", code);
            return UniReturnUtil.success(data);
        }
        return UniReturnUtil.fail("没有配置验证码规则");
    }


    //用户登录

    public Map<String, Object> login(Map<String, Object> requestData) throws Exception {

        Optional<String> username = getValue("account", requestData);
        Optional<String> password = getValue("password", requestData);
        Optional<String> verifycode = getValue("verifycode", requestData);

        String appId = RequestUtil.getAppId();
        String sessionId = RequestUtil.getSessionId();
        String ip = RequestUtil.getIpAddr();
//        解密验证码
        Map<String, Object> application = RequestUtil.getApplication();
        Object securityCodeRule = application.get("securitycoderule");
        String code = null;

        if (Objects.nonNull(securityCodeRule) && !debug) {
            code = DataEncryptionUtil.decryptRSAByPrivateKey(verifycode.get());
            if (Objects.nonNull(securityCodeRule) && !check(code, sessionId, appId, ip)) {
                return UniReturnUtil.fail("验证码错误");
            }
        }

        String query = "select * from userinfo where  account =? and userpassword=? ";
        List<Map<String, Object>> userInfoList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{username.get(), DataEncryptionUtil.decryptRSAByPrivateKey(password.get())});
        }});
        if (userInfoList.isEmpty()) {
            return UniReturnUtil.fail("用户名密码错误");
        }
        Map<String, Object> userInfo = userInfoList.get(0);
        Object userId = userInfo.get("userid");
        RequestContextHolder.currentRequestAttributes().setAttribute("userid", userId, SCOPE_SESSION);
        RequestContextHolder.currentRequestAttributes().setAttribute("userinfo", userInfo, SCOPE_SESSION);
        String query1 = "select * from userloginlog where userid=?";
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, query1, new ArrayList<>() {{
            add(new Object[]{userInfo.get("userid")});
        }});

        Map<String, Object> data = new HashMap<>();
        String appToken = RequestUtil.getAppToken();
        String insert = "insert into userloginlog ( userid, requestip, sessionid, logintime, usertoken, lasttime, lastheartbeat,apptoken,isexpires,appid)values (?,?,?,?,?,?,?,?,0,?)";
        if (userLoginLogList.isEmpty()) {
            data.put("userstatus", "0");
            DataBase.updateBatch(Config.securityConnectionStr, insert, new ArrayList<>() {{
                add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
            }});
        } else {
            Object multilogin = application.get("multilogin");
            if (Objects.equals(multilogin, "1")) {
                Optional<Map<String, Object>> log = userLoginLogList.stream().filter(it -> it.get("sessionid").equals(sessionId)).findAny();
                if (log.isEmpty()) {
                    DataBase.updateBatch(Config.securityConnectionStr, insert, new ArrayList<>() {{
                        add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
                    }});
                }
                data.put("userstatus", "0");

            } else {
                Optional<Map<String, Object>> log = userLoginLogList.stream().filter(it -> it.get("sessionid").equals(sessionId)).findAny();
                if (log.isEmpty()) {

                    DataBase.updateBatch(Config.securityConnectionStr, insert, new ArrayList<>() {{
                        add(new Object[]{userId, ip, sessionId, LocalDateTime.now(), null, LocalDateTime.now(), LocalDateTime.now(), appToken, appId});
                    }});
                }
                data.put("userstatus", "1");
            }
        }
        data.put("userid", userId);
        if (Objects.nonNull(securityCodeRule)) {
            remove(code, sessionId, appId, ip);
        }
        return UniReturnUtil.success(data);
    }


    //强制登录

    public Map<String, Object> forceLogin() throws Exception {

        Map<String, Object> application = RequestUtil.getApplication();

        String appToken = RequestUtil.getAppToken();
        String ip = RequestUtil.getIpAddr();
        String sessionId = RequestUtil.getSessionId();
        String query = "select * from userloginlog where apptoken=? and sessionid=? and requestip=? and isexpires=0 ";
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{appToken, sessionId, ip});
        }});
        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("登录失败");
        }
        Map<String, Object> userLoginLog = userLoginLogList.get(0);
        Object securityCodeEffectiveObj = application.get("securitycodeeffective");
        Long securityCodeEffective = defaultSecurityCodeEffective;
        if (Objects.nonNull(securityCodeEffectiveObj)) {
            securityCodeEffective = Long.parseLong(securityCodeEffectiveObj.toString());
        }
        LocalDateTime expiresTime = LocalDateTime.now().plusSeconds(securityCodeEffective);
        String userToken = DataEncryptionUtil.signatureMD5("%s:%s".formatted(LocalDateTime.now(), sessionId));
        String update = "update userloginlog set apptoken=null,usertoken=?,lasttime=? where  loginid=?";
        DataBase.updateBatch(Config.securityConnectionStr, update, new ArrayList<>() {{
            add(new Object[]{userToken, expiresTime, userLoginLog.get("loginid")});
        }});
        HashMap<String, Object> data = new HashMap<>();
        data.put("expirestime", expiresTime);
        data.put("usertoken", userToken);

        return UniReturnUtil.success(data);

    }

    public Map<String, Object> checkUserToken(String userToken) throws Exception {
        String sessionId = RequestUtil.getSessionId();
        String query = "select * from userloginlog where isexpires = 0 and usertoken = ? and sessionid = ?";
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{userToken, sessionId});
        }});
        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("用户登录查询失败");
        }
        Map<String, Object> userLoginLog = userLoginLogList.get(0);
        Object lastheartbeat = userLoginLog.get("lastheartbeat");
        LocalDateTime dateTime = DataFormatUtil.toDateTime(lastheartbeat);
        if (Objects.nonNull(dateTime)) {
            Map<String, Object> application = RequestUtil.getApplication();
            Object apptokeneffectiveObj = application.get("apptokeneffective");
            Long apptokeneffective = defaultAppTokenEffective;
            if (Objects.nonNull(apptokeneffectiveObj)) {
                apptokeneffective = Long.parseLong(apptokeneffectiveObj.toString());
            }

            if (dateTime.plusSeconds(apptokeneffective).isAfter(LocalDateTime.now())) {
                return UniReturnUtil.success(null);
            }
        }
        return UniReturnUtil.fail("用户token已过期");

    }

    //用户登出

    public Map<String, Object> logOut() throws Exception {
        String userToken = RequestUtil.getUserToken();
        String sessionId = RequestUtil.getSessionId();
        String query = "select * from userloginlog where isexpires=0 and usertoken=? and sessionid=?";

        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{userToken, sessionId});
        }});

        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("登出失败");
        }
        Map<String, Object> userLoginLog = userLoginLogList.get(0);
        Object userIdObj = userLoginLog.get("userid");
        String delete = "delete from userloginlog where  userid=?";
        DataBase.updateBatch(Config.securityConnectionStr, delete, new ArrayList<>() {{
            add(new Object[]{userIdObj});
        }});


        return UniReturnUtil.success("成功");
    }


    //获取用户权限

    public Map<String, Object> permission() throws Exception {

        String userId = RequestUtil.getUserId();
        String query = """
                                select userpermissionsid,
                       t2.userid,
                       t3.*,
                       t1.serviceid,
                       insetcolumnlist,
                       updatecolumnlist,
                       selectcolumnlist,
                       filterset
                from pageconfiguration t3,
                     userpermissions t1,
                          userinfo t2
                where t1.userid = t2.userid
                  and  t3.pageconfigurationid = t1.pageconfigurationid
                  and t1.userid = ?""";
        List<Map<String, Object>> permission = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{userId});
        }});
        return UniReturnUtil.success(permission);
    }

    //应用API及数据权限
    public Map<String, Object> changePassword(Map<String, Object> requestData) throws Exception {
        //       Todo
        Optional<String> oldPasswordOpt = getValue("oldpassword", requestData);
        Optional<String> passwordOpt = getValue("password", requestData);
        Map<String, Object> userInfo = RequestUtil.getUserInfo();
        String userpassword = userInfo.get("userpassword").toString();
        String password = DataEncryptionUtil.decryptRSAByPrivateKey(passwordOpt.get());
        String oldPassword = DataEncryptionUtil.decryptRSAByPrivateKey(oldPasswordOpt.get());
        if (!userpassword.equals(oldPassword)) {
            return UniReturnUtil.fail("密码错误");
        } else {
            String userId = RequestUtil.getUserId();
            String update = "update userinfo set userpassword=? where userid=?";
            DataBase.updateBatch(Config.securityConnectionStr, update, new ArrayList<>() {{
                add(new Object[]{password, userId});
            }});
            return UniReturnUtil.success("成功");
        }

    }

    //用户心跳
    public Map<String, Object> userHeartbeat() throws Exception {

        String userToken = RequestUtil.getUserToken();
        String sessionId = RequestUtil.getSessionId();
        String query = "select * from userloginlog where isexpires=0 and usertoken=? and sessionid=?";
        List<Map<String, Object>> userLoginLogList = DataBase.query(Config.securityConnectionStr, query, new ArrayList<>() {{
            add(new Object[]{userToken, sessionId});
        }});
        if (userLoginLogList.isEmpty()) {
            return UniReturnUtil.fail("查询失败");
        } else {
            return UniReturnUtil.success("查询成功");
        }

    }

    public void addCode(String code, String sessionId, String appid, Long securitycodeeffective, String requestIp) throws Exception {
//      使用数据库
        LocalDateTime localDateTime = LocalDateTime.now().plusSeconds(securitycodeeffective);
        String insertSql = "insert into tempsecuritycode(appid,requestip,sessionid,securitycode,expiretime) values (?,?,?,?,?)";
        DataBase.updateBatch(Config.securityConnectionStr, insertSql, new ArrayList<>() {{
            add(new Object[]{appid, requestIp, sessionId, code, localDateTime});
        }});
        String deleteSql = "delete from tempsecuritycode where expiretime < ?";
        DataBase.updateBatch(Config.securityConnectionStr, deleteSql, new ArrayList<>() {{
            add(new Object[]{LocalDateTime.now()});
        }});
    }

    public boolean check(String code, String sessionId, String appid, String requestIp) throws Exception {

        String selectSql = "select * from tempsecuritycode where securitycode=? and sessionid=? and appid=? and requestip=?";
        List<Map<String, Object>> map = DataBase.query(Config.securityConnectionStr, selectSql, new ArrayList<>() {{
            add(new Object[]{code, sessionId, appid, requestIp});
        }});
        return !map.isEmpty();

    }

    public void remove(String code, String sessionId, String appid, String requestIp) throws Exception {
        String deleteSql = "delete from tempsecuritycode where securitycode=? and sessionid=? and appid=? and requestip=?";
        DataBase.updateBatch(Config.securityConnectionStr, deleteSql, new ArrayList<>() {{
            add(new Object[]{code, sessionId, appid, requestIp});
        }});

    }

}
