package com.scbfkj.uni.system;

import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.service.DataProcessService;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;

import java.util.*;
import java.util.concurrent.*;

public class ScheduleUtil {


    private static ThreadPoolTaskScheduler threadPoolTaskScheduler = new ThreadPoolTaskScheduler();


    private static Map<String, ScheduledFuture<?>> scheduledFutureMap = new HashMap<>();

    private final static Map<String, Map<String, Object>> serviceInfoesMap = new HashMap<>();
    public static Map<String, ScheduleTask> scheduleTaskMaps = new HashMap<>();

    static {
        threadPoolTaskScheduler.setPoolSize(8);
        threadPoolTaskScheduler.initialize();
        System.out.println("定时任务线程池启动");
    }

    /**
     * 启动
     *
     * @param serviceId 服务id
     */
    public static boolean start(String serviceId) throws Exception {

        if (!serviceInfoesMap.containsKey(serviceId)) {
            //采集数据源及参数配置
            List<Map<String, Object>> serviceInfos = DataBase.query(Config.centerConnectionStr, "SELECT SI.* FROM  serviceinfo SI WHERE SI.servicetype ='4'  AND serviceid = ?", Collections.singletonList(new Object[]{serviceId}));

            if (Objects.isNull(serviceInfos) || serviceInfos.isEmpty()) {
                throw new RuntimeException("采集服务不存在");
            }
            Map<String, Object> serviceInfo = serviceInfos.get(0);
            serviceInfoesMap.put(serviceId, serviceInfo);
        }

        Map<String, Object> serviceInfo = serviceInfoesMap.get(serviceId); //采集服务

        Integer loopCount = Objects.isNull(serviceInfo.get("loopcount")) ? null : Integer.parseInt(serviceInfo.get("loopcount").toString());
        Object cronExpress = serviceInfo.get("cronexpress");
        if (Objects.isNull(cronExpress)) {
            throw new RuntimeException("服务:%s，不是定时任务");
        }
        if (!scheduleTaskMaps.containsKey(serviceId)) {
            ScheduleTask scheduleTask = new ScheduleTask(serviceId, loopCount);
            scheduleTaskMaps.put(serviceId, scheduleTask);
        }
        ScheduleTask scheduleTask = scheduleTaskMaps.get(serviceId);
        //创建定时任务：
        System.out.println("启动定时任务线程 taskId " + scheduleTask.getId());// 设置时间比当前时间晚了
        if (Objects.isNull(cronExpress)) {
//                todo 记录日志
            throw new RuntimeException("定时表达式并没有设置");
        }
        ScheduledFuture<?> scheduledFuture = threadPoolTaskScheduler.schedule(scheduleTask, new CronTrigger(cronExpress.toString()));
        scheduledFutureMap.put(serviceId, scheduledFuture);
        DataBase.updateBatch(Config.centerConnectionStr, "update serviceinfo set runState = 1 where  serviceid =?", Collections.singletonList(new Object[]{serviceId}));
        return true;
    }


    public static boolean start(Runnable runnable,String cron){
        threadPoolTaskScheduler.schedule(runnable, new CronTrigger(cron));
        return true;
    }
    /**
     * 取消，停止定时任务
     *
     * @param serviceId 定时任务
     */
    public static boolean cancel(String serviceId) throws Exception {
        System.out.println("关闭定时任务线程 taskId " + serviceId);
        ScheduledFuture<?> scheduledFuture = scheduledFutureMap.remove(serviceId);
        if (scheduledFuture != null && !scheduledFuture.isCancelled()) {
            scheduledFuture.cancel(false);
            DataBase.updateBatch(Config.centerConnectionStr, "update serviceinfo set runState = 0 where  serviceID =?", Collections.singletonList(new Object[]{serviceId}));
        }
        return true;
    }
}
