create table apiinfo
(
    apiid       int auto_increment comment '自增长主键'
        primary key,
    apiname     varchar(255) null comment 'api名称',
    apitype     varchar(255) null comment '类型（外部api、内部方法、服务）',
    requestpath varchar(255) null comment '请求路径',
    routepath   varchar(255) null comment '转发路径（外部api完整路径、带类名的内部方法名、服务编号）'
);

create table appconnectlog
(
    connid      int auto_increment comment '自增主键'
        primary key,
    appid       varchar(16) null comment '应用编码',
    requesttime datetime    null comment '连接时间',
    requestip   varchar(32) null comment '请求ip',
    apptoken    varchar(64) null comment 'app连接token',
    expiretime  datetime    null comment 'token过期时间',
    lasttime    datetime    null comment '最后活跃时间'
);

create table application
(
    applicationid         int auto_increment comment '自增主键'
        primary key,
    appid                 varchar(16)  null comment '应用编码',
    appsecret             varchar(32)  null comment '应用密钥',
    appname               varchar(64)  null comment '应用名称',
    appengname            varchar(128) null comment '应用英文名称',
    appdescribe           varchar(256) null comment '应用描述',
    applogo               varchar(256) null comment '应用图标',
    smalllogo             varchar(256) null comment '应用小图标',
    backgroundimage       varchar(256) null comment '背景图',
    apptokeneffective     int          null comment '应用令牌有效时长(分钟，从连接起)',
    securitycoderule      varchar(128) null comment '验证码规则',
    securitycodeeffective int          null comment '验证码有效时长(秒)',
    multilogin            int          null comment '是否允许多机登录',
    passwordrule          varchar(128) null comment '密码规则',
    passwordeffective     int          null comment '密码过期时长'
);

create table applicationpermissions
(
    applicationpermissionsid int auto_increment comment '自增长主键应用api权限编号'
        primary key,
    appid                    int          null comment 'app编号',
    apiid                    int          null comment 'api编号',
    columnlist               varchar(200) null comment '列权限',
    filterset                varchar(200) null comment '行权限'
);

create table breakerrule
(
    breakerruleid        int auto_increment comment '自增长主键熔断策略编号'
        primary key,
    breakerrulename      varchar(50)   null comment '策略名',
    duration             int           null comment '单位时间（秒）',
    limitvalue           int           null comment '流量上限',
    averageerrorrate     decimal(5, 2) null comment '平均错误率',
    errorcount           int           null comment '错误次数',
    averageexecutiontime int           null comment '平均执行时长',
    returnmessage        varchar(100)  null comment '超限熔断返回提示'
);

create table container
(
    containerid       int auto_increment comment '自增长主键容器编号'
        primary key,
    containercode     varchar(50)  null comment '运行实例编号',
    containername     varchar(50)  null comment '名称',
    containerdescribe varchar(200) null comment '用途说明',
    requesturi        varchar(200) null comment '访问路径',
    containervesion   varchar(50)  null comment '版本号',
    runstate          varchar(20)  null comment '当前状态',
    lasttime          datetime     null comment '最后活跃时间'
);

create table datasource
(
    datasourceid       int auto_increment comment '自增长主键'
        primary key,
    datasourcedescribe varchar(200) null comment '数据源说明',
    connectset         varchar(200) null comment '连接字符串'
);

create table pageconfiguration
(
    pageconfigurationid int auto_increment comment '自增长主键界面配置编号'
        primary key,
    pagetype            varchar(50)  null comment '类型（模块、页面、按钮、表格）',
    superiorid          int          null comment '页面配置上级编号',
    pagename            varchar(50)  null comment '名称',
    pagecode            varchar(50)  null comment '界面控件编码',
    pagedescribe        varchar(200) null comment '描述',
    serviceid           int          null comment '绑定服务编号'
);

create table ratelimitrule
(
    ratelimitingid   int auto_increment comment '自增长主键限流策略编号'
        primary key,
    ratelimitingname varchar(50)  null comment '策略名',
    duration         int          null comment '单位时间（秒）',
    limitvalue       int          null comment '流量上限',
    returnmessage    varchar(100) null comment '超限保护返回提示',
    apiid            varchar(100) null comment 'api编号'
);

create table servicestate
(
    servicestateid int auto_increment comment '自增长主键'
        primary key,
    serviceid      int         null comment '服务编号',
    containerid    int         null comment '容器编号',
    containercode  varchar(50) null comment '运行实例编号',
    starttime      datetime    null comment '启动时间',
    stoptime       datetime    null comment '停止时间',
    runstate       varchar(20) null comment '当前状态',
    lasttime       datetime    null comment '最后活跃时间'
);

create table tableconfiguration
(
    tableconfigurationid int auto_increment comment '自增长主键表格配置编号'
        primary key,
    pagecode             varchar(50)  null,
    columnname           varchar(50)  null comment '列名',
    columncode           varchar(50)  null comment '列标识',
    groupname            varchar(50)  null comment '上级分组名称',
    columndescribe       varchar(200) null comment '列描述',
    datatype             varchar(50)  null comment '数据类型',
    datalength           int          null comment '数据长度',
    isdisplay            int          null comment '是否显示',
    displaynumber        int          null comment '显示顺序',
    displaywidth         int          null comment '显示宽度',
    enablesort           int          null comment '启用排序',
    enablefilter         int          null comment '启用过滤',
    enablegroup          int          null comment '启用分组',
    enablecount          int          null comment '启用计算',
    isfiltercolumn       int          null comment '是否为过滤列',
    isprimarykey         int          null comment '是否为主键',
    defaultparameters    varchar(200) null comment '默认查询参数'
);

create table tempsecuritycode
(
    codeid       int auto_increment comment '自增主键'
        primary key,
    appid        varchar(16) null comment '应用编码',
    requestip    varchar(32) null comment '请求ip',
    sessionid    varchar(64) null comment '连接sessionid',
    securitycode varchar(8)  null comment '验证码',
    expiretime   datetime    null comment '过期时间'
);

create table usergroup
(
    usergroupid       int auto_increment comment '自增长主键用户组编号'
        primary key,
    superiorid        int          null comment '上级用户组编号',
    usergroupname     varchar(50)  null comment '名称',
    usergroupdescribe varchar(200) null comment '描述'
);

create table usergrouppermissions
(
    userpermissionsid   int auto_increment comment '自增长主键用户组权限编号'
        primary key,
    usergroupid         int          null comment '用户组编号',
    pageconfigurationid int          null comment '页面配置编号',
    serviceid           int          null comment '服务编号',
    insertcolumnlist    varchar(200) null comment '列权限插入列表',
    updatecolumnlist    varchar(200) null comment '列权限更新列表',
    selectcolumnlist    varchar(200) null comment '列权限查询列表',
    filterset           varchar(200) null comment '行权限过滤条件'
);

create table userinfo
(
    userid                    int auto_increment comment '自增长主键用户编号'
        primary key,
    usergroupid               int          null comment '用户组编号',
    username                  varchar(50)  null comment '用户名',
    account                   varchar(50)  null comment '账号',
    userpassword              varchar(50)  null comment '密码',
    userdescribe              varchar(200) null comment '描述',
    isenable                  int          null comment '是否启用账号',
    secondarypassword         varchar(50)  null comment '二级安全密码',
    lastestchangepasswordtime datetime     null comment '最后修改密码时间'
);

create table userloginlog
(
    loginid       int auto_increment comment '自增长主键登录编号'
        primary key,
    userid        int          null,
    requestip     varchar(50)  null comment '请求地址',
    sessionid     varchar(50)  null comment '请求编号',
    logintime     datetime     null comment '登录时间',
    usertoken     varchar(100) null comment '用户令牌',
    lasttime      datetime     null comment '最后活跃时间',
    lastheartbeat datetime     null comment '最后心跳时间',
    logouttime    datetime     null comment '登出时间',
    apptoken      varchar(128) null comment '连接令牌',
    isexpires     int          null comment '是否过期',
    appid         varchar(16)  null comment 'app编号',
    expirestime   datetime     null comment '过期时间'
);

create table userpermissions
(
    userpermissionsid   int auto_increment comment '自增长主键用户权限编号'
        primary key,
    userid              int          null comment '用户编号',
    pageconfigurationid int          null comment '页面配置编号',
    serviceid           int          null comment '服务编号',
    insertcolumnlist    varchar(200) null comment '列权限插入列表',
    updatecolumnlist    varchar(200) null comment '列权限更新列表',
    selectcolumnlist    varchar(200) null comment '列权限查询列表',
    filterset           varchar(200) null comment '行权限过滤条件'
);

