package com.scbfkj.uni.api;

import com.jayway.jsonpath.JsonPath;
import com.scbfkj.uni.ControllerBaseTest;
import org.hamcrest.core.IsEqual;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
class SecurityApiTest extends ControllerBaseTest {
    public void testHello() throws Exception {

        /*
         * 1、mockMvc.perform执行一个请求。
         * 2、MockMvcRequestBuilders.get("XXX")构造一个请求。
         * 3、ResultActions.param添加请求传值
         * 4、ResultActions.accept(MediaType.TEXT_HTML_VALUE))设置返回类型
         * 5、ResultActions.andExpect添加执行完成后的断言。
         * 6、ResultActions.andDo添加一个结果处理器，表示要对结果做点什么事情
         *   比如此处使用MockMvcResultHandlers.print()输出整个响应结果信息。
         * 7、ResultActions.andReturn表示执行完成后返回相应的结果。
         */
        mockMvc.perform(MockMvcRequestBuilders
                        .get("/hello")
                        // 设置返回值类型为utf-8，否则默认为ISO-8859-1
                        .accept(MediaType.APPLICATION_JSON_UTF8_VALUE)
                        .param("name", "Tom"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string("Hello Tom!"))
                .andDo(MockMvcResultHandlers.print());
    }


    @Test
    @Order(1)
    void getToken() throws Exception {

        mockMvc.perform(MockMvcRequestBuilders.post("/user/getToken")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
                        .content("""
                                {"appid":"test","appsecret":"test"}
                                """)
                        .cookie(cookie))

                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.jsonPath("code", IsEqual.equalTo("0")))
                .andDo(result -> {
                    String content = result.getResponse().getContentAsString();
                    SecurityApiTest.token = JsonPath.read(content, "$.returnData.token");
                })
                .andDo(MockMvcResultHandlers.print());
    }

    @Test
    @Order(2)
    void refreshToken() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.post("/user/refreshToken")
                        .contentType(MediaType.APPLICATION_JSON_VALUE)
//                        .header("token", token)
                        .cookie(cookie)
                                .content("")
                )

                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.jsonPath("code", IsEqual.equalTo("0")))
                .andDo(MockMvcResultHandlers.print());
    }

    @Test
    void testToken() {
    }

    @Test
    void getCode() {
    }

    @Test
    void forceLogin() {
    }

    @Test
    void login() {
    }

    @Test
    void getPermissions() {
    }

    @Test
    void changePwd() {
    }

    @Test
    void logOut() {
    }

    @Test
    void health() {
    }

    @Test
    void getPubKey() {
    }
}